@@ -15,14 +15,13 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- *  $Id$
  */
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
 
-#include <gtk/gtktoolitem.h>
+#include <gtk/gtk.h>
 #include <glib/gi18n.h>
 
 #include "ephy-topic-factory-action.h"
