@@ -1,3 +1,4 @@
+/* -*- Mode: C; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 8; -*- */
 /*
  *  Copyright © 2004 Tommi Komulainen
  *  Copyright © 2004, 2005 Christian Persch
@@ -16,39 +17,26 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  *
- *  $Id$
  */
 
 #include "config.h"
 
+#include "ephy-embed-utils.h"
 #include "ephy-find-toolbar.h"
-#include "ephy-embed-find.h"
-#include "ephy-embed-factory.h"
 #include "ephy-debug.h"
 
+#include <webkit/webkit.h>
 #include <gdk/gdkkeysyms.h>
 #include <glib/gi18n.h>
 #include <gtk/gtk.h>
-#include <gtk/gtkarrow.h>
-#include <gtk/gtkcheckbutton.h>
-#include <gtk/gtkhbox.h>
-#include <gtk/gtkimage.h>
-#include <gtk/gtkentry.h>
-#include <gtk/gtklabel.h>
-#include <gtk/gtkseparatortoolitem.h>
-#include <gtk/gtkstock.h>
-#include <gtk/gtktoolbutton.h>
-#include <gtk/gtkalignment.h>
-#include <gtk/gtkmain.h>
 #include <string.h>
 
 #define EPHY_FIND_TOOLBAR_GET_PRIVATE(object) (G_TYPE_INSTANCE_GET_PRIVATE ((object),EPHY_TYPE_FIND_TOOLBAR, EphyFindToolbarPrivate))
 
 struct _EphyFindToolbarPrivate
 {
-	EphyEmbedFind *find;
 	EphyWindow *window;
-	EphyEmbed *embed;
+	WebKitWebView *web_view;
 	GtkWidget *entry;
 	GtkWidget *label;
 	GtkToolItem *next;
@@ -60,6 +48,7 @@ struct _EphyFindToolbarPrivate
 	gulong set_focus_handler;
 	guint source_id;
 	guint find_again_source_id;
+	char *find_string;
 	guint preedit_changed : 1;
 	guint prevent_activate : 1;
 	guint activated : 1;
@@ -81,27 +70,45 @@ enum
 	LAST_SIGNAL
 };
 
+typedef enum
+{
+	EPHY_FIND_FOUND		= 0,
+	EPHY_FIND_NOTFOUND	= 1,
+	EPHY_FIND_FOUNDWRAPPED	= 2
+} EphyEmbedFindResult;
+
 static guint signals[LAST_SIGNAL];
 
 /* private functions */
 
-static EphyEmbedFind *
-get_find (EphyFindToolbar *toolbar)
+static void
+scroll_lines (WebKitWebView *web_view,
+              int num_lines)
 {
-	EphyFindToolbarPrivate *priv = toolbar->priv;
+        GtkScrolledWindow *scrolled_window;
+        GtkAdjustment *vadj;
+        gdouble value;
 
-	if (priv->find == NULL)
-	{
-		LOG ("Creating the finder now");
+        scrolled_window = GTK_SCROLLED_WINDOW (gtk_widget_get_parent (GTK_WIDGET (web_view)));
+        vadj = gtk_scrolled_window_get_vadjustment (scrolled_window);
 
-		priv->find = EPHY_EMBED_FIND (ephy_embed_factory_new_object (EPHY_TYPE_EMBED_FIND));
+        value = vadj->value + (num_lines * vadj->step_increment);
+        gtk_adjustment_set_value (vadj, value);
+}
 
-		g_return_val_if_fail (priv->embed == NULL || GTK_WIDGET_REALIZED (GTK_WIDGET (priv->embed)), priv->find);
+static void
+scroll_pages (WebKitWebView *web_view,
+              int num_pages)
+{
+        GtkScrolledWindow *scrolled_window;
+        GtkAdjustment *vadj;
+        gdouble value;
 
-		ephy_embed_find_set_embed (priv->find, priv->embed);
-	}
+        scrolled_window = GTK_SCROLLED_WINDOW (gtk_widget_get_parent (GTK_WIDGET (web_view)));
+        vadj = gtk_scrolled_window_get_vadjustment (scrolled_window);
 
-	return priv->find;
+        value = vadj->value + (num_pages * vadj->page_increment);
+        gtk_adjustment_set_value (vadj, value);
 }
 
 static gboolean
@@ -109,15 +116,15 @@ set_status_notfound_cb (EphyFindToolbar *toolbar)
 {
 	EphyFindToolbarPrivate *priv;
 	PangoFontDescription *pango_desc = NULL;
-	
+
 	priv = toolbar->priv;
-	
+
 	pango_desc = pango_font_description_new ();
 	gtk_widget_modify_font (priv->status_label, pango_desc);
 	pango_font_description_free (pango_desc);
-	
+
 	priv->source_id = 0;
-	
+
 	return FALSE;
 }
 
@@ -131,10 +138,10 @@ set_status (EphyFindToolbar *toolbar,
 
 	switch (result)
 	{
-		case EPHY_EMBED_FIND_FOUND:
+		case EPHY_FIND_FOUND:
 			text = NULL;
 			break;
-		case EPHY_EMBED_FIND_NOTFOUND:
+		case EPHY_FIND_NOTFOUND:
 			{
 				text = _("Not found");
 
@@ -142,12 +149,12 @@ set_status (EphyFindToolbar *toolbar,
 				pango_font_description_set_weight (pango_desc, PANGO_WEIGHT_BOLD);
 				gtk_widget_modify_font (priv->status_label, pango_desc);
 				pango_font_description_free (pango_desc);
-				
+
 				gtk_widget_error_bell (GTK_WIDGET (priv->window));
 				priv->source_id = g_timeout_add (500, (GSourceFunc) set_status_notfound_cb, toolbar);
 			}
 			break;
-		case EPHY_EMBED_FIND_FOUNDWRAPPED:
+		case EPHY_FIND_FOUNDWRAPPED:
 			text = _("Wrapped");
 			break;
 	}
@@ -171,6 +178,7 @@ clear_status (EphyFindToolbar *toolbar)
 			    priv->links_only ? _("Find links:") : _("Find:"));
 }
 
+#if 0
 /* Code adapted from gtktreeview.c:gtk_tree_view_key_press() and
  * gtk_tree_view_real_start_interactive_seach()
  */
@@ -204,6 +212,7 @@ tab_search_key_press_cb (EphyEmbed *embed,
 
 	return FALSE;
 }
+#endif
 
 static void
 find_next_cb (EphyFindToolbar *toolbar)
@@ -217,6 +226,74 @@ find_prev_cb (EphyFindToolbar *toolbar)
 	g_signal_emit (toolbar, signals[PREVIOUS], 0);
 }
 
+static void
+set_string_and_highlight (EphyFindToolbarPrivate *priv, const char *find_string)
+{
+        WebKitWebView *web_view = priv->web_view;
+        gboolean case_sensitive;
+
+        if (g_strcmp0 (priv->find_string, find_string) != 0) {
+                g_free (priv->find_string);
+                priv->find_string = g_strdup (find_string);
+        }
+
+        case_sensitive = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (priv->case_sensitive));
+
+        webkit_web_view_unmark_text_matches (web_view);
+        webkit_web_view_mark_text_matches (web_view,
+                                           priv->find_string,
+                                           case_sensitive,
+                                           0);
+        webkit_web_view_set_highlight_text_matches (web_view, TRUE);
+}
+
+static void
+ephy_find_toolbar_set_properties (EphyFindToolbar *toolbar,
+                                  const char *find_string)
+{
+        EphyFindToolbarPrivate *priv = toolbar->priv;
+
+        set_string_and_highlight (priv, find_string);
+}
+
+static EphyEmbedFindResult
+real_find (EphyFindToolbarPrivate *priv,
+	   gboolean forward)
+{
+        WebKitWebView *web_view = priv->web_view;
+        gboolean case_sensitive = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (priv->case_sensitive));
+
+        if (!priv->find_string || !g_strcmp0 (priv->find_string, ""))
+                return EPHY_FIND_NOTFOUND;
+
+        if (!webkit_web_view_search_text
+            (web_view, priv->find_string, case_sensitive, forward, FALSE)) {
+                /* not found, try to wrap */
+                if (!webkit_web_view_search_text
+                    (web_view, priv->find_string, case_sensitive, forward, TRUE)) {
+                        /* there's no result */
+                        return EPHY_FIND_NOTFOUND;
+                } else {
+                        /* found wrapped */
+                        return EPHY_FIND_FOUNDWRAPPED;
+                }
+        }
+
+        return EPHY_FIND_FOUND;
+}
+
+static EphyEmbedFindResult
+ephy_find_toolbar_find (EphyFindToolbar *toolbar,
+                        const char *find_string,
+                        gboolean links_only)
+{
+        EphyFindToolbarPrivate *priv = toolbar->priv;
+
+        set_string_and_highlight (priv, find_string);
+
+        return real_find (priv, TRUE);
+}
+
 static void
 entry_changed_cb (GtkEntry *entry,
 		  EphyFindToolbar *toolbar)
@@ -224,17 +301,22 @@ entry_changed_cb (GtkEntry *entry,
 	EphyFindToolbarPrivate *priv = toolbar->priv;
 	const char *text;
 	EphyEmbedFindResult result;
-	gboolean case_sensitive;
 
 	text = gtk_entry_get_text (GTK_ENTRY (priv->entry));
-	case_sensitive = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (priv->case_sensitive));
 
-	ephy_embed_find_set_properties (get_find (toolbar), text, case_sensitive);
-	result = ephy_embed_find_find (get_find (toolbar), text, priv->links_only);
+	ephy_find_toolbar_set_properties (toolbar, text);
+	result = ephy_find_toolbar_find (toolbar, text, priv->links_only);
 
 	set_status (toolbar, result);
 }
 
+static gboolean
+ephy_find_toolbar_activate_link (EphyFindToolbar *toolbar,
+                                 GdkModifierType mask)
+{
+        return FALSE;
+}
+
 static gboolean
 entry_key_press_event_cb (GtkEntry *entry,
 			  GdkEventKey *event,
@@ -250,20 +332,20 @@ entry_key_press_event_cb (GtkEntry *entry,
 		switch (event->keyval)
 		{
 		case GDK_Up:
-                case GDK_KP_Up:
-			ephy_embed_scroll (priv->embed, -1);
+		case GDK_KP_Up:
+			scroll_lines (priv->web_view, -1);
 			break;
 		case GDK_Down:
-                case GDK_KP_Down:
-			ephy_embed_scroll (priv->embed, 1);
+		case GDK_KP_Down:
+			scroll_lines (priv->web_view, 1);
 			break;
 		case GDK_Page_Up:
-                case GDK_KP_Page_Up:
-			ephy_embed_page_scroll (priv->embed, -1);
+		case GDK_KP_Page_Up:
+			scroll_pages (priv->web_view, -1);
 			break;
 		case GDK_Page_Down:
-                case GDK_KP_Page_Down:
-			ephy_embed_page_scroll (priv->embed, 1);
+		case GDK_KP_Page_Down:
+			scroll_pages (priv->web_view, 1);
 			break;
 		case GDK_Escape:
 			/* Hide the toolbar when ESC is pressed */
@@ -279,7 +361,7 @@ entry_key_press_event_cb (GtkEntry *entry,
 		  event->keyval == GDK_KP_Enter ||
 		  event->keyval == GDK_ISO_Enter))
 	{
-		handled = ephy_embed_find_activate_link (get_find (toolbar), event->state);
+		handled = ephy_find_toolbar_activate_link (toolbar, event->state);
 	}
 	else if ((event->state & mask) == GDK_SHIFT_MASK &&
 		 (event->keyval == GDK_Return ||
@@ -301,7 +383,7 @@ entry_activate_cb (GtkWidget *entry,
 
 	if (priv->typing_ahead)
 	{
-		ephy_embed_find_activate_link (get_find (toolbar), 0);
+		ephy_find_toolbar_activate_link (toolbar, 0);
 	}
 	else
 	{
@@ -348,7 +430,7 @@ case_sensitive_toggled_cb (GtkWidget *check,
 			(proxy, G_CALLBACK (case_sensitive_menu_toggled_cb), toolbar);
 	}
 
-	ephy_embed_find_set_properties (get_find (toolbar), text, case_sensitive);
+	ephy_find_toolbar_set_properties (toolbar, text);
 }
 
 static gboolean
@@ -495,7 +577,7 @@ ephy_find_toolbar_init (EphyFindToolbar *toolbar)
 
 	priv->sep = gtk_separator_tool_item_new ();
 	gtk_toolbar_insert (GTK_TOOLBAR (toolbar), priv->sep, -1);
-	
+
 	priv->status_item = gtk_tool_item_new ();
 	gtk_tool_item_set_expand (priv->status_item, TRUE);
 	priv->status_label = gtk_label_new ("");
@@ -528,12 +610,6 @@ ephy_find_toolbar_dispose (GObject *object)
 	EphyFindToolbar *toolbar = EPHY_FIND_TOOLBAR (object);
 	EphyFindToolbarPrivate *priv = toolbar->priv;
 
-	if (priv->find != NULL)
-	{
-		g_object_unref (priv->find);
-		priv->find = NULL;
-	}
-	
 	if (priv->source_id != 0)
 	{
 		g_source_remove (priv->source_id);
@@ -575,13 +651,23 @@ ephy_find_toolbar_set_property (GObject *object,
 	}
 }
 
+static void
+ephy_find_toolbar_finalize (GObject *o)
+{
+  EphyFindToolbarPrivate *priv = EPHY_FIND_TOOLBAR (o)->priv;
+
+  g_free (priv->find_string);
+  G_OBJECT_CLASS (ephy_find_toolbar_parent_class)->finalize (o);
+}
+
 static void
 ephy_find_toolbar_class_init (EphyFindToolbarClass *klass)
 {
 	GObjectClass *object_class = G_OBJECT_CLASS (klass);
-        GtkWidgetClass *widget_class = GTK_WIDGET_CLASS (klass);
+	GtkWidgetClass *widget_class = GTK_WIDGET_CLASS (klass);
 
 	object_class->dispose = ephy_find_toolbar_dispose;
+	object_class->finalize = ephy_find_toolbar_finalize;
 	object_class->get_property = ephy_find_toolbar_get_property;
 	object_class->set_property = ephy_find_toolbar_set_property;
 
@@ -636,7 +722,7 @@ EphyFindToolbar *
 ephy_find_toolbar_new (EphyWindow *window)
 {
 	return EPHY_FIND_TOOLBAR (g_object_new (EPHY_TYPE_FIND_TOOLBAR,
-				  		"window", window,
+						"window", window,
 						NULL));
 }
 
@@ -653,30 +739,28 @@ ephy_find_toolbar_set_embed (EphyFindToolbar *toolbar,
 			     EphyEmbed *embed)
 {
 	EphyFindToolbarPrivate *priv = toolbar->priv;
+	WebKitWebView *web_view = EPHY_GET_WEBKIT_WEB_VIEW_FROM_EMBED(embed);
 
-	if (priv->embed == embed) return;
+	if (priv->web_view == web_view) return;
 
-	if (priv->embed != NULL)
+	if (priv->web_view != NULL)
 	{
+                /*
 		g_signal_handlers_disconnect_matched (embed, G_SIGNAL_MATCH_DATA,
 						      0, 0, NULL, NULL, toolbar);
+                */
 	}
 
-	priv->embed = embed;
-	if (embed != NULL)
+	priv->web_view = web_view;
+	if (web_view != NULL)
 	{
 		clear_status (toolbar);
 
+		/* FIXME: this is never emitted
 		g_signal_connect_object (embed, "ge-search-key-press",
 					 G_CALLBACK (tab_search_key_press_cb),
 					 toolbar, 0);
-
-		if (priv->find != NULL)
-		{
-			g_return_if_fail (GTK_WIDGET_REALIZED (GTK_WIDGET (priv->embed)));
-
-			ephy_embed_find_set_embed (priv->find, embed);
-		}
+                */
 	}
 }
 
@@ -691,7 +775,16 @@ find_again_data_destroy_cb (FindAgainCBStruct *data)
 {
 	g_slice_free (FindAgainCBStruct, data);
 }
-	
+
+static EphyEmbedFindResult
+ephy_find_toolbar_find_again (EphyFindToolbar *toolbar,
+                              gboolean forward,
+                              gboolean links_only)
+{
+  EphyFindToolbarPrivate *priv = toolbar->priv;
+
+  return real_find (priv, forward);
+}
 
 static gboolean
 find_again_cb (FindAgainCBStruct *data)
@@ -699,12 +792,12 @@ find_again_cb (FindAgainCBStruct *data)
 	EphyEmbedFindResult result;
 	EphyFindToolbarPrivate *priv = data->toolbar->priv;
 
-	result = ephy_embed_find_find_again (get_find (data->toolbar), data->next,
-					     priv->links_only);
+	result = ephy_find_toolbar_find_again (data->toolbar, data->next,
+				  priv->links_only);
 	set_status (data->toolbar, result);
-    
+
 	priv->find_again_source_id = 0;
-    
+
 	return FALSE;
 }
 
@@ -724,13 +817,13 @@ ephy_find_toolbar_find_next (EphyFindToolbar *toolbar)
 	 * to sync with the size change due to the toolbar being shown, otherwise
 	 * the toolbar can obscure the result. See GNOME bug #415074.
 	 */
-    
+
 	if (priv->find_again_source_id != 0) return;
 
-    	data = g_slice_new0 (FindAgainCBStruct);
+	data = g_slice_new0 (FindAgainCBStruct);
 	data->toolbar = toolbar;
 	data->next = TRUE;
-    
+
 	priv->find_again_source_id = g_idle_add_full (G_PRIORITY_DEFAULT_IDLE,
 						      (GSourceFunc) find_again_cb,
 						      data,
@@ -748,13 +841,13 @@ ephy_find_toolbar_find_previous (EphyFindToolbar *toolbar)
 		gtk_widget_show (widget);
 		gtk_widget_grab_focus (widget);
 	}
-    
+
 	if (priv->find_again_source_id != 0) return;
 
 	data = g_slice_new0 (FindAgainCBStruct);
 	data->toolbar = toolbar;
 	data->next = FALSE;
-    
+
 	priv->find_again_source_id = g_idle_add_full (G_PRIORITY_DEFAULT_IDLE,
 						      (GSourceFunc) find_again_cb,
 						      data,
@@ -768,7 +861,7 @@ ephy_find_toolbar_open (EphyFindToolbar *toolbar,
 {
 	EphyFindToolbarPrivate *priv = toolbar->priv;
 
-	g_return_if_fail (priv->embed != NULL);
+	g_return_if_fail (priv->web_view != NULL);
 
 	priv->typing_ahead = typing_ahead;
 	priv->links_only = links_only;
@@ -781,6 +874,15 @@ ephy_find_toolbar_open (EphyFindToolbar *toolbar,
 	gtk_widget_grab_focus (GTK_WIDGET (toolbar));
 }
 
+static void
+ephy_find_toolbar_set_selection (EphyFindToolbar *toolbar,
+                                 gboolean attention)
+{
+        WebKitWebView *web_view = toolbar->priv->web_view;
+
+        webkit_web_view_set_highlight_text_matches (web_view, attention);
+}
+
 void
 ephy_find_toolbar_close (EphyFindToolbar *toolbar)
 {
@@ -788,8 +890,8 @@ ephy_find_toolbar_close (EphyFindToolbar *toolbar)
 
 	gtk_widget_hide (GTK_WIDGET (toolbar));
 
-	if (priv->embed == NULL || priv->find == NULL) return;
-	ephy_embed_find_set_selection (get_find (toolbar), FALSE);
+	if (priv->web_view == NULL) return;
+	ephy_find_toolbar_set_selection (toolbar, FALSE);
 }
 
 void
