@@ -17,12 +17,12 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- *  $Id$
  */
 
 #include "config.h"
 
 #include "ephy-embed-container.h"
+#include "ephy-embed-utils.h"
 #include "ephy-history.h"
 #include "ephy-location-action.h"
 #include "ephy-location-entry.h"
@@ -52,6 +52,7 @@ struct _EphyLocationActionPrivate
 	guint editable : 1;
 	guint show_lock : 1;
 	guint secure : 1;
+	gboolean sync_address_is_blocked;
 };
 
 static void ephy_location_action_init       (EphyLocationAction *action);
@@ -175,11 +176,20 @@ action_activated_cb (GtkEntryCompletion *completion,
 
 static void
 entry_activate_cb (GtkEntry *entry,
-		   EphyLocationAction *action)
+		   EphyLocationAction *proxy)
 {
 	EphyBookmarks *bookmarks;
 	const char *content;
 	char *address;
+	GtkAction *action;
+	
+	action = gtk_activatable_get_related_action (GTK_ACTIVATABLE (proxy));
+
+	if (EPHY_LOCATION_ACTION (action)->priv->sync_address_is_blocked)
+	{
+		EPHY_LOCATION_ACTION (action)->priv->sync_address_is_blocked = FALSE;
+		g_signal_handlers_unblock_by_func (action, G_CALLBACK (sync_address), proxy);
+	}	
 
 	content = gtk_entry_get_text (entry);
 	if (content == NULL || content[0] == '\0') return;
@@ -189,7 +199,7 @@ entry_activate_cb (GtkEntry *entry,
 	address = ephy_bookmarks_resolve_address (bookmarks, content, NULL);
 	g_return_if_fail (address != NULL);
 
-	ephy_link_open (EPHY_LINK (action), address, NULL, 
+	ephy_link_open (EPHY_LINK (action), g_strstrip (address), NULL, 
 		        ephy_link_flags_from_current_event () | EPHY_LINK_ALLOW_FIXUP);
 
 	g_free (address);
@@ -316,7 +326,7 @@ get_location_cb (EphyLocationEntry *entry,
 	embed = ephy_embed_container_get_active_child 
           (EPHY_EMBED_CONTAINER (priv->window));
 
-	return g_strdup (ephy_embed_get_address (embed));
+	return g_strdup (ephy_web_view_get_address (EPHY_GET_EPHY_WEB_VIEW_FROM_EMBED (embed)));
 }
 
 static char *
@@ -328,7 +338,7 @@ get_title_cb (EphyLocationEntry *entry,
 	embed = ephy_embed_container_get_active_child 
           (EPHY_EMBED_CONTAINER (action->priv->window));
 
-	return g_strdup (ephy_embed_get_title (embed));
+	return g_strdup (ephy_web_view_get_title (EPHY_GET_EPHY_WEB_VIEW_FROM_EMBED (embed)));
 }
 
 static void
@@ -382,6 +392,40 @@ add_completion_actions (GtkAction *gaction,
 			  G_CALLBACK (action_activated_cb), action);
 }
 
+static gboolean
+focus_in_event_cb (GtkWidget *entry,
+		   GdkEventFocus *event,
+		   GtkWidget *proxy)
+{
+	GtkAction *action;
+	action = gtk_activatable_get_related_action (GTK_ACTIVATABLE (proxy));
+
+	if (!EPHY_LOCATION_ACTION (action)->priv->sync_address_is_blocked)
+	{		
+		EPHY_LOCATION_ACTION (action)->priv->sync_address_is_blocked = TRUE;
+		g_signal_handlers_block_by_func (action, G_CALLBACK (sync_address), proxy);
+	}
+	
+	return FALSE;
+}
+
+static gboolean
+focus_out_event_cb (GtkWidget *entry,
+		    GdkEventFocus *event,
+		    GtkWidget *proxy)
+{
+	GtkAction *action;
+	action = gtk_activatable_get_related_action (GTK_ACTIVATABLE (proxy));
+
+	if (EPHY_LOCATION_ACTION (action)->priv->sync_address_is_blocked)
+	{
+		EPHY_LOCATION_ACTION (action)->priv->sync_address_is_blocked = FALSE;
+		g_signal_handlers_unblock_by_func (action, G_CALLBACK (sync_address), proxy);
+	}
+	
+	return FALSE;
+}
+
 static void
 connect_proxy (GtkAction *action, GtkWidget *proxy)
 {
@@ -402,8 +446,8 @@ connect_proxy (GtkAction *action, GtkWidget *proxy)
 						    EPHY_COMPLETION_EXTRA_COL,
 						    EPHY_COMPLETION_FAVICON_COL);
 		
-		ephy_location_entry_set_match_func (EPHY_LOCATION_ENTRY (proxy),
-							match_func,
+		ephy_location_entry_set_match_func (EPHY_LOCATION_ENTRY (proxy), 
+							match_func, 
 							proxy,
 							NULL);
 
@@ -434,7 +478,7 @@ connect_proxy (GtkAction *action, GtkWidget *proxy)
 		entry = ephy_location_entry_get_entry (lentry);
 		g_signal_connect_object (entry, "activate",
 					 G_CALLBACK (entry_activate_cb),
-					 action, 0);
+					 proxy, 0);
 		g_signal_connect_object (proxy, "user-changed",
 					 G_CALLBACK (user_changed_cb), action, 0);
 		g_signal_connect_object (proxy, "lock-clicked",
@@ -443,6 +487,10 @@ connect_proxy (GtkAction *action, GtkWidget *proxy)
 					 G_CALLBACK (get_location_cb), action, 0);
 		g_signal_connect_object (proxy, "get-title",
 					 G_CALLBACK (get_title_cb), action, 0);
+		g_signal_connect_object (entry, "focus-in-event",
+					 G_CALLBACK (focus_in_event_cb), proxy, 0);
+		g_signal_connect_object (entry, "focus-out-event",
+					 G_CALLBACK (focus_out_event_cb), proxy, 0);
 	}
 
 	GTK_ACTION_CLASS (ephy_location_action_parent_class)->connect_proxy (action, proxy);
@@ -758,6 +806,7 @@ ephy_location_action_init (EphyLocationAction *action)
 	priv->bookmarks = ephy_shell_get_bookmarks (ephy_shell);
 	priv->smart_bmks = ephy_bookmarks_get_smart_bookmarks
 		(action->priv->bookmarks);
+	priv->sync_address_is_blocked = FALSE;
 
 	init_actions_list (action);
 	
