@@ -16,14 +16,13 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- *  $Id: ephy-shell.c 7927 2008-02-10 18:26:22Z xan $
  */
 
 #include "config.h"
 
 #include <string.h>
 #include <glib/gi18n.h>
-#include <gtk/gtknotebook.h>
+#include <gtk/gtk.h>
 #include <dirent.h>
 #include <unistd.h>
 
@@ -32,10 +31,13 @@
 #include "ephy-embed-container.h"
 #include "ephy-embed-shell.h"
 #include "ephy-embed-single.h"
+#include "ephy-embed-utils.h"
 #include "eel-gconf-extensions.h"
 #include "ephy-prefs.h"
 #include "ephy-file-helpers.h"
 #include "ephy-favicon-cache.h"
+#include "ephy-web-view.h"
+#include "ephy-embed-utils.h"
 #include "ephy-window.h"
 #include "ephy-bookmarks-ui.h"
 #include "ephy-bookmarks-import.h"
@@ -54,7 +56,7 @@
 #include "ephy-prefs.h"
 #include "ephy-gui.h"
 #include "ephy-stock-icons.h"
-#include "ephy-embed-factory.h"
+#include "ephy-web-view.h"
 
 #ifdef ENABLE_NETWORK_MANAGER
 #include "ephy-net-monitor.h"
@@ -109,7 +111,7 @@ ephy_shell_class_init (EphyShellClass *klass)
 static EphyEmbed *
 ephy_shell_new_window_cb (EphyEmbedSingle *single,
 			  EphyEmbed *parent_embed,
-			  EphyEmbedChrome chromemask,
+			  EphyWebViewChrome chromemask,
 			  EphyShell *shell)
 {
 	GtkWidget *parent = NULL;
@@ -123,7 +125,7 @@ ephy_shell_new_window_cb (EphyEmbedSingle *single,
 
 	if (eel_gconf_get_boolean (CONF_LOCKDOWN_DISABLE_JAVASCRIPT_CHROME))
 	{
-		chromemask = EPHY_EMBED_CHROME_ALL;
+		chromemask = EPHY_WEB_VIEW_CHROME_ALL;
 	}
 
 	if (parent_embed != NULL)
@@ -138,7 +140,7 @@ ephy_shell_new_window_cb (EphyEmbedSingle *single,
 	/* what's a popup ? ATM, any window opened with menubar toggled on 
    	 * is *not* a popup 
 	 */
-	is_popup = (chromemask & EPHY_EMBED_CHROME_MENUBAR) == 0;
+	is_popup = (chromemask & EPHY_WEB_VIEW_CHROME_MENUBAR) == 0;
 
 	return ephy_shell_new_tab_full
 		(shell,
@@ -394,8 +396,7 @@ load_homepage (EphyEmbed *embed)
 	}
 
 	is_empty = url_is_empty (home);
-
-	ephy_embed_load_url (embed, home);
+	ephy_web_view_load_url (EPHY_WEB_VIEW (EPHY_GET_WEBKIT_WEB_VIEW_FROM_EMBED (embed)), home);
 
 	g_free (home);
 
@@ -407,7 +408,7 @@ load_homepage (EphyEmbed *embed)
  * @shell: a #EphyShell
  * @parent_window: the target #EphyWindow or %NULL
  * @previous_embed: the referrer embed, or %NULL
- * @url: an url to load or %NULL
+ * @request: a #WebKitNetworkRequest to load or %NULL
  * @chrome: a #EphyEmbedChrome mask to use if creating a new window
  * @is_popup: whether the new window is a popup
  * @user_time: a timestamp, or 0
@@ -421,9 +422,9 @@ EphyEmbed *
 ephy_shell_new_tab_full (EphyShell *shell,
 			 EphyWindow *parent_window,
 			 EphyEmbed *previous_embed,
-			 const char *url,
+			 WebKitNetworkRequest *request,
 			 EphyNewTabFlags flags,
-			 EphyEmbedChrome chrome,
+			 EphyWebViewChrome chrome,
 			 gboolean is_popup,
 			 guint32 user_time)
 {
@@ -473,7 +474,7 @@ ephy_shell_new_tab_full (EphyShell *shell,
 		embed = ephy_embed_container_get_active_child (EPHY_EMBED_CONTAINER (window));
 		if (embed != NULL)
 		{
-			if (ephy_embed_get_is_blank (embed))
+			if (ephy_web_view_get_is_blank (EPHY_GET_EPHY_WEB_VIEW_FROM_EMBED (embed)))
 			{
 				active_is_blank = TRUE;
 			}
@@ -481,7 +482,7 @@ ephy_shell_new_tab_full (EphyShell *shell,
 	}
 	if (active_is_blank == FALSE)
 	{
-		embed = EPHY_EMBED (ephy_embed_factory_new_object (EPHY_TYPE_EMBED));
+		embed = EPHY_EMBED (g_object_new (EPHY_TYPE_EMBED, NULL));
 		g_assert (embed != NULL);
 		gtk_widget_show (GTK_WIDGET (embed));
 		
@@ -490,11 +491,8 @@ ephy_shell_new_tab_full (EphyShell *shell,
 
 	if (previous_embed != NULL)
 	{	
-		ephy_embed_shistory_copy (previous_embed,
-					  embed,
-					  TRUE,   /* back history */
-					  TRUE,   /* forward history */
-					  FALSE); /* current index */
+                ephy_web_view_copy_back_history (EPHY_WEB_VIEW (EPHY_GET_WEBKIT_WEB_VIEW_FROM_EMBED (previous_embed)),
+                                                 EPHY_WEB_VIEW (EPHY_GET_WEBKIT_WEB_VIEW_FROM_EMBED (embed)));
 	}		
 
 	ephy_gui_window_update_user_time (GTK_WIDGET (window), user_time);
@@ -512,32 +510,17 @@ ephy_shell_new_tab_full (EphyShell *shell,
 	if (flags & EPHY_NEW_TAB_HOME_PAGE ||
 	    flags & EPHY_NEW_TAB_NEW_PAGE)
 	{
-		ephy_embed_set_typed_address (embed, "",
-                                              EPHY_EMBED_ADDRESS_EXPIRE_NEXT);
+		ephy_web_view_set_typed_address (EPHY_GET_EPHY_WEB_VIEW_FROM_EMBED (embed), "",
+						 EPHY_WEB_VIEW_ADDRESS_EXPIRE_NEXT);
 		ephy_toolbar_activate_location (toolbar);
 		is_empty = load_homepage (embed);
 	}
 	else if (flags & EPHY_NEW_TAB_OPEN_PAGE)
 	{
-		EphyEmbedLoadFlags load_flags = 0;
+                ephy_web_view_load_request (EPHY_WEB_VIEW (EPHY_GET_WEBKIT_WEB_VIEW_FROM_EMBED (embed)),
+                                            request);
 
-		g_assert (url != NULL);
-
-		if (flags & EPHY_NEW_TAB_ALLOW_FIXUP)
-		{
-			load_flags = EPHY_EMBED_LOAD_FLAGS_ALLOW_THIRD_PARTY_FIXUP;
-		}
-		else
-		{
-			load_flags = EPHY_EMBED_LOAD_FLAGS_NONE;
-		}
-		/* FIXME */
-		/* We need to audit every caller to see if this 
-		   won't make us send referer for undesirable loads.
-	           Passing NULL referrer atm  */
-		ephy_embed_load (embed, url, load_flags, NULL);
-
-		is_empty = url_is_empty (url);
+		is_empty = url_is_empty (webkit_network_request_get_uri (request));
 	}
 
         /* Make sure the initial focus is somewhere sensible and not, for
@@ -583,9 +566,15 @@ ephy_shell_new_tab (EphyShell *shell,
 		    const char *url,
 		    EphyNewTabFlags flags)
 {
-	return ephy_shell_new_tab_full (shell, parent_window,
-					previous_embed, url, flags,
-					EPHY_EMBED_CHROME_ALL, FALSE, 0);
+	EphyEmbed *embed;
+	WebKitNetworkRequest *request = webkit_network_request_new (url);
+
+	embed = ephy_shell_new_tab_full (shell, parent_window,
+					 previous_embed, request, flags,
+					 EPHY_WEB_VIEW_CHROME_ALL, FALSE, 0);
+        g_object_unref (request);
+
+	return embed;
 }
 
 /**
